//-----------------------------------------------
// To compile, 'make mocables all'.
//-----------------------------------------------
#include <qapplication.h>
#include <qmessagebox.h>
#include <qobject.h>
#include <qbitmap.h>
#include <qimage.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qtextstream.h>
#include <qfiledialog.h>
#include <qtimer.h>
//-----------------------------------------------
#include "BmpToSrc.h"
//-----------------------------------------------
// Minimal objects created for connectivity
MyApp::MyApp ( QObject * )
    : vLayout ( this )
    , listBox ( this )
    , leName ( this )
    , lePackage ( this ) {
    bVerbose = false;
    pbSave = NULL;
    connect ( &leName, 
              SIGNAL ( textChanged (const QString&) ), 
              this, 
              SLOT ( slotNameChanged (const QString&) ) ) ;
    connect ( &lePackage, 
              SIGNAL ( textChanged (const QString&) ), 
              this, 
              SLOT ( slotPackageChanged (const QString&) ) ) ;
}
//-----------------------------------------------
void MyApp::setupQt () {
	QRect rect;
	QSize btnSize   ( 100, 30 );
	QSize labelSize (  80, 30 );

	QHBoxLayout * layName    = new QHBoxLayout ();
	QHBoxLayout * layPackage = new QHBoxLayout ();
	QHBoxLayout * layButtons = new QHBoxLayout ();

	setCaption ( "Bitmap to Src Converter" );
	rect.setSize ( labelSize );

// Lables and entry fields
	lName = new QLabel ( "Name : ", this );
	lName->setFixedSize ( labelSize );
	layName->addWidget ( lName );
	layName->addWidget ( &leName );

	lPackage = new QLabel ( "Package : ", this );
	lPackage->setFixedSize ( labelSize );
	layPackage->addWidget ( lPackage );
	layPackage->addWidget ( &lePackage );

// ListBox
	listBox.resize ( 200, 500 );

// Buttons
	pbSave = new QPushButton ( this );
	pbSave->setFixedSize ( btnSize );
	pbSave->setText ( "Save" );
	pbSave->setEnabled ( false );
	layButtons->addWidget ( pbSave );

	pbQuit = new QPushButton ( this );
	pbQuit->setText ( "Quit" );
	pbQuit->setFixedSize ( btnSize );
	layButtons->addWidget ( pbQuit );

// Add these things at this point !
	vLayout.addLayout ( layName );
	vLayout.addLayout ( layPackage );
	vLayout.addWidget ( &listBox );
	vLayout.addLayout ( layButtons );

	connect ( pbSave, 
				SIGNAL ( clicked () ), 
				this, 
				SLOT ( slotSave () ) ) ;
	connect ( pbQuit, 
				SIGNAL ( clicked () ), 
				this, 
				SLOT ( slotQuit () ) ) ;
}
//-----------------------------------------------
// Request filename
int MyApp::loadFile () {
	inFile = QFileDialog::getOpenFileName (
    	                QDir::homeDirPath (),
        	            "Images (*.bmp)",
            	        NULL,
                	    "Bmp2Src",
                    	"Select bitmap to convert" );
    if ( bVerbose ) {
        QString s = QString ( "Bitmap file '%1' selected" ).arg ( inFile );
        qDebug ( s );
    } /* endif */
    return process ();
}
//-----------------------------------------------
int MyApp::process () {
    if ( ! QFile::exists ( inFile ) ) {
        if ( bVerbose ) {
            QString s = QString ( "Bitmap file '%1' not found" ).arg ( inFile );
            qDebug ( s );
        } /* endif */
        return eBitmapNotOpened;
    }
	bmp = QBitmap ( inFile );			// Load bitmap
    if ( bmp.isNull () ) {
        if ( bVerbose ) {
            QString s = QString ( "File '%1' is not a bitmap" ).arg ( inFile );
            qDebug ( s );
        } /* endif */
        return eInfileNotBitmap;
    }
    writeBitmapToStream ();
    if ( pbSave ) pbSave->setEnabled ( true );
    return eNoError;
}
//-----------------------------------------------
void MyApp::textChanged ( QLineEdit& le,
						  int        iLine ) {
	QString s;

	s = le.text ();
	s.truncate ( 16 );
	le.setText ( s );
	s.prepend ( '\'' );
	s.append ( '\'' );
	listBox.changeItem ( s, iLine );
}
//-----------------------------------------------
void MyApp::slotNameChanged (const QString&) {
	textChanged ( leName, 0 );
}
//-----------------------------------------------
void MyApp::slotPackageChanged (const QString&) {
	textChanged ( lePackage, 1 );
}
//-----------------------------------------------
void MyApp::slotSave () {
	QFile 	file;
	bool	bRc;

	changeExtension ();
	do {
		file.setName ( outFile );
		bRc = file.open ( IO_WriteOnly );
		if ( bRc ) break;
// Request a new filename
		outFile = QFileDialog::getSaveFileName (
						QDir::homeDirPath (),
						"Source (*.src)",
						NULL,
						"Bmp2Src",
						"Output File" );
		if ( outFile.isEmpty () ) return;
	} while ( ! bRc );
	listBoxToFile ( file );
}
//-----------------------------------------------
void MyApp::listBoxToFile ( QFile& file ) {
	QTextStream 	stream;
	QListBoxText *	lbi;
	QString			s;

    if ( bVerbose ) {
        s = QString ( "Writing file '%1'" ).arg ( file.name () );
        qDebug ( s );
    } /* endif */
    stream.setDevice (&file);
	for ( unsigned int uiLine = 0; uiLine < listBox.count (); uiLine++ ) {
		lbi = (QListBoxText *) listBox.item (uiLine);
		s = lbi->text ();
		stream << s << "\n";
	} /* endfor */stream << "\n";
	file.close ();
}
//-----------------------------------------------
void MyApp::slotQuit () {
	close ();
}
//-----------------------------------------------
void MyApp::append ( QString& s ) {
	new QListBoxText ( &listBox, s );
}
//-----------------------------------------------
void MyApp::append ( const char * psz ) {
	QString s ( psz );
	new QListBoxText ( &listBox, s );
}
//-----------------------------------------------
void MyApp::writeBitmapToStream () {
	QImage 			image;
	unsigned char 	ucMask;
	unsigned char 	ucData = 0;
	int				iRowCount;
	int 			iColBlocks;
	int 			iRowBlocks;
	unsigned int 	uiByteIndex = 0;
	int				iColCount;
	unsigned char * ucLineBuf;
	QString 		s;
	
	s = QString ( "'%1'" ).arg ( leName.text () );
	append ( s );

    s = QString ( "'%1'" ).arg ( lePackage.text () );
	append ( s );

// now, we have to break up the data line by line	
    if ( bVerbose ) qDebug ( "Converting QBitmap to QImage" );
    image = bmp.convertToImage ();
	iRowCount = bmp.height ();
	iColCount = bmp.width  ();
	iColBlocks = (iColCount - 1) / 10;
	iRowBlocks = (iRowCount - 1) / 10;

	s = QString ( "%1 %2 1" ).arg ( iColBlocks).arg ( iRowBlocks );
	append ( s );
	s = "BITMAP";
	append ( s );

    if ( bVerbose ) qDebug ( "Reading scanlines into QListBox" );
    for ( int iRow = 0; iRow < iRowCount; iRow++ ) {
		s = QString ( "{%1}" ).arg ( iRow, 3 );
		ucMask = 0;
		ucLineBuf = image.scanLine ( iRow );
		uiByteIndex = 0;
		for ( int iCol = 0; 
			  iCol < iColCount; 
			  iCol++, ucMask <<= 1 ) {			// Assumes MSB first
			if ( ucMask == 0 ) {
				ucMask = 0x01;					
				ucData = ucLineBuf [uiByteIndex++];
			} /* endif */
			if ( ucData & ucMask ) 
				s.append ( '#' );
			
			else
				s.append ( '.' );
			
		} /* endfor */
		append ( s );
	} /* endfor */
    if ( bVerbose ) {
        s = QString ( "%1 Scanlines read into QListBox" ).arg ( iRowCount );
        qDebug ( s );
    } /* endif */
    return;
}
//-----------------------------------------------
bool MyApp::setInFile  ( QString& infile ) {
    if ( bVerbose ) {
        QString s = QString ( "Setting input bitmap to '%1'" ).arg ( infile );
        qDebug ( s );
    } /* endif */
    inFile = infile;
	return process ();
}
//-----------------------------------------------
QString& MyApp::changeExtension () {
	QStringList list;

// Change '.bmp' extension to '.src'
	list = QStringList::split ( '.', inFile );
	list.last () = QString ( "src" );
	outFile = list.join ( "." );
    if ( bVerbose ) {
        QString s = QString ( "Output file set to to '%1'" ).arg ( outFile );
        qDebug ( s );
    } /* endif */
    return outFile;
}
//-----------------------------------------------
int MyApp::cmdLine ( int argc, char ** argv ) {
	QFile 		 file;
	bool 		 bRc;
	int			 iFileCount = 0;
	QStringList  list;
	QString 	 s;
	QString 	 sKey;
	QString 	 sVal;

    for ( int iArg = 1; iArg < argc; iArg++ ) {
		s = QString ( argv [iArg] );
		list = QStringList::split ( '=', s );
		switch ( list.count () ) {
		case 1:							// filenames
            if ( s == "-v" ) {
                bVerbose = true;
                continue;
            } /* endif */
            if ( s == "-h" ) {
                help ();
                return eNoError;
            } /* endif */
            switch ( iFileCount++ ) {
			case eInFile:
				bRc = setInFile ( s );
                if ( bRc ) {
                    if ( bVerbose ) {
                        QString s = QString ( "Bitmap file '%1' cannot be opened" ).arg ( inFile );
                        qDebug ( s );
                    } /* endif */
                    return eBitmapNotOpened;
				} /* endif */
				s = changeExtension ();
				file.setName ( s );
				continue;

			case eOutFile:
				file.setName ( s );
                continue;
			} /* endswitch */
			qDebug ( "Too many files" );
			return eTooManyFiles;					// too many file parameters

		case 2:							// marks it as name=val
			sKey = list [0];
			sVal = list [1];
			if ( sKey == "-name" ) {
				leName.setText ( sVal );
                continue;
			} /* endif */
            if ( sKey == "-package" ) {
                lePackage.setText ( sVal );
                continue;
            } /* endif */
            s = QString ( "Unknown key - '%1'" ).arg ( sKey );
			qDebug ( s );
			return eUnknownKey;					// unknown key

		default:
			s = QString ( "Impure key - '%1'" ).arg ( sKey );
			qDebug ( s );
			return eImpureKey;					// 'a=b=c' key
			break;
		} /* endswitch */
	} /* endfor */
    
	bRc = file.open ( IO_WriteOnly );
	if ( !bRc ) {
		qDebug ( "Output file cannot be created" );
		return eOutfileNotCreated;
	} /* endif */
	listBoxToFile ( file );
	return eNoError;
}
//-----------------------------------------------
void help () {
	qDebug ( "bmp2src : Converts monochrome bitmaps to schedit library source files" );
    qDebug ( "bmp2src - Qt graphics program. Usage : " );
	qDebug ( "	'>bmp2src [-v] [-key=val][-key=val] <infile> [outfile]'" );
	qDebug ( "	where keys are 'name' and package'" );
	qDebug ( "	infile describes a monochrome bitmap file" );
	qDebug ( "	outfile describes a library source file" );
    qDebug ( "	If 'outfile' is not given, then the extension of 'infile' is converted to '.src'" );
    qDebug ( "	The '-v' flag puts the program in verbose mode" );
    qDebug ( "	'>bmp2src' -h gives this screen, regardless of any other parameters" );
}
//-----------------------------------------------
int main ( int argc, char ** argv ) {
	int			 iRc = 0;	
	QApplication app ( argc, argv );

  	MyApp myapp;

// forms of operation - command line parms prevent windowing system operating
	switch ( argc ) {
	case 1:										// 'Qt' windowing required
		myapp.setupQt ();
		iRc = myapp.loadFile ();			// 'Cancel' called
        if ( iRc ) break;
		iRc = myapp.exec ();
		break;

	default:
// test for 'help' invokation
		iRc = myapp.cmdLine ( argc, argv );
		return iRc;
	} /* endswitch */	
	return iRc;
}
//-----------------------------------------------

#include "BmpToSrc.moc"

